#include "bloc.h"

#include <iostream>
using namespace std;

#include <GL/glut.h>

Bloc::Bloc(float longueur, float largeur, float hauteur, Point3D position, RVBA couleur)
: longueur(longueur), largeur(largeur), hauteur(hauteur), Objet(position, couleur)
{
}

void Bloc::afficher()
{
    glShadeModel(GL_FLAT); //ne lisse pas le bloc
    glEnable(GL_NORMALIZE); //normalisation des vecteurs automatiques
    glEnableClientState(GL_NORMAL_ARRAY); //active les tableaux de normales
    glEnableClientState(GL_VERTEX_ARRAY); //active les tableaux de sommets
    
    glEnable(GL_CULL_FACE); //active l'effacement des faces caches
    glFrontFace(GL_CW); //definit la face visible
    glCullFace(GL_FRONT); //on cache certaines faces (les faces devants)    

    glColor4f(this->couleur.r, this->couleur.v, this->couleur.b, this->couleur.a);

    GLfloat mat_specular[] = {0.0, 0.0, 0.0, 1.0};
    GLfloat mat_amb_diff[] = {0.5, 0.5, 0.5, 1.0};

    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat_amb_diff); 

    glPushMatrix(); //sauvegarde la matrice courante
    
    //on se positionne la ou le bloc va tre dessin
    glTranslatef(this->position.x, this->position.y, this->position.z);
    
    //cout << "affichage bloc : " << this->position.x << " " << this->position.y << " " << this->position.z << endl;
    //cout << this->couleur.r << " " << this->couleur.v << " " << this->couleur.b << " " << this->couleur.a << endl;
    
    //les coordonnes des sommets du bloc (8 sommets)
    GLdouble sommets[24];
    
    //le sommet n0
    sommets[0] = 0.0; sommets[1] = 0.0; sommets[2] = 0.0;
    //le sommet n1
    sommets[3] = 0.0; sommets[4] = 0.0; sommets[5] = this->hauteur;
    //le sommet n2
    sommets[6] = 0.0; sommets[7] = this->largeur; sommets[8] = 0.0;
    //le sommet n3
    sommets[9] = 0.0; sommets[10] = this->largeur; sommets[11] = this->hauteur;
    
    //le sommet n4
    sommets[12] = this->longueur; sommets[13] = 0.0; sommets[14] = 0.0;
    //le sommet n5
    sommets[15] = this->longueur; sommets[16] = 0.0; sommets[17] = this->hauteur;
    //le sommet n6
    sommets[18] = this->longueur; sommets[19] = this->largeur; sommets[20] = 0.0;
    //le sommet n7
    sommets[21] = this->longueur; sommets[22] = this->largeur; sommets[23] = this->hauteur;    
    
    //les normales du bloc
    GLint normales[] = {-1,0,0 , 0,0,1 , 1,0,0 , 0,0,-1 , 0,-1,0 , 0,1,0};
   
    glVertexPointer(3, GL_DOUBLE, 0, sommets); //associe le pointeur de sommet sur les sommets du cube
    glNormalPointer(GL_INT, 0, normales); //associe le pointeur de normales sur chaques normale

    //definition de l'ordre de tracage des polygones
    GLubyte indices[] = {0,1,3,2,  5,7,3,1,  5,4,6,7,  4,0,2,6,  1,0,4,5,  3,7,6,2};
    glDrawElements(GL_QUADS, 24, GL_UNSIGNED_BYTE, indices);
    
    glPopMatrix(); //restaure la matrice
}

int Bloc::verifCollision(Point3D A)
{
    int positionValide = 0;

    //cout << "A.x: " << A.x << "\n";
    //cout << "A.z: " << A.z << "\n";
    //cout << "position.x: " << this->position.x << "\n";
    //cout << "longueur: " << this->longueur << "\n";
    //cout << "position.z: " << this->position.z << "\n";
    //cout << "largeur: " << this->hauteur << "\n";

    if((A.x > this->position.x) && (A.x < this->position.x + this->longueur) && (A.z > this->position.z) && (A.z < this->position.z + this->hauteur) && (this->position.y < 2.0))
    {
        positionValide = 1;
    }

    return positionValide;
}
