#include "cible.h"
#include "types.h"

#include <iostream>
using namespace std;

#define _WCHAR_T_DEFINED
#define _WCHAR_T_
#include <GL/glut.h>

Cible::Cible(Point3D position, RVBA couleur) : Objet(position, couleur)
{
    this->touche = false;
    
    //la hauteur et la largeur du poteau
    this->potHauteur = 1;
    this->potLargeur = 0.2;
        
    //la hauteur et la largeur de la cible
    this->ciHauteur = 1;
    this->ciLargeur = 1;
}

void Cible::calculerImpact(Point3D depuis, Vecteur3D direction)
{
    /*
    calcul la position de l'impact, les coordonnes
    (lambda, mu, gamma) sont relatives au rectangle form par la cible
    */
  
    //tolerance du zero
    const double TOLERANCE_ZERO = 0.0001;
                
    //les vecteurs e et f dans le plan
    //ATTENTION : il sont calculs que pour une certaine orientation de la cible
    Vecteur3D e(0, 0, 1);
    Vecteur3D f(0, 1, 0);
    
    //le vecteur sur la droite (normalis) 
    double longD = sqrt(pow(direction.x, 2) + pow(direction.y, 2) + pow(direction.z, 2));
    Vecteur3D d(direction.x/longD, direction.y/longD, direction.z/longD);
    
    //un point sur la droite
    Point3D A = depuis;
    
    //un point dans le plan
    Point3D B(
        0+this->position.x,
        this->potHauteur+this->position.y,
        -this->ciLargeur/2+this->position.z);
    
    double lambdaMuDenominateur =
        d.z*(-e.y*f.x + e.x*f.y) +
        d.y*( e.z*f.x - e.x*f.z) +
        d.x*(-e.z*f.y + e.y*f.z);
        
    if(lambdaMuDenominateur > -TOLERANCE_ZERO & lambdaMuDenominateur < TOLERANCE_ZERO) return;
    
    double lambda =
       (-A.y*e.z*f.x + B.y*e.z*f.x + A.x*e.z*f.y - B.x*e.z*f.y +
         A.z*( e.y*f.x - e.x*f.y) +
         B.z*(-e.y*f.x + e.x*f.y) +
         A.y*e.x*f.z - B.y*e.x*f.z - A.x*e.y*f.z + B.x*e.y*f.z) / lambdaMuDenominateur;
         
         
    double mu = 
       (A.y*d.z*e.x + B.y*d.z*e.x - A.x*d.z*e.y + B.x*d.z*e.y +
        B.z*( d.y*e.x - d.x*e.y) +
        A.z*(-d.y*e.x + d.x*e.y) -
        A.y*d.x*e.z + B.y*d.x*e.z + A.x*d.y*e.z - B.x*d.y*e.z) / lambdaMuDenominateur;
        
    double gammaDenominateur = d.x*(lambdaMuDenominateur);
    
    if(gammaDenominateur > -TOLERANCE_ZERO & gammaDenominateur < TOLERANCE_ZERO) return;
    
    double gamma =
       ((( A.z - B.z)*d.x + (-A.x + B.x)*d.z)* (d.y*f.x - d.x*f.y) +
         (A.y*d.x - B.y*d.x + (-A.x + B.x)*d.y)*(-d.z*f.x + d.x*f.z)) / gammaDenominateur;

    //test si l'impact est sur la cible
    if(lambda > 0 && mu > 0 && mu < this->ciLargeur && gamma > 0 && gamma < this->ciHauteur)
        this->touche = true;
    
    //cout << "lambda : " << lambda << "  mu : " << mu << "  gamma : " << gamma << endl;      
}

void Cible::afficher()
{
    glShadeModel(GL_FLAT); //ne lisse pas les cibles
    glEnable(GL_NORMALIZE); //normalisation des vecteurs automatiques
    
    glDisable(GL_CULL_FACE); //dsacive l'effacement des faces caches
    
    glColor4f(this->touche ? 0 : this->couleur.r,
              this->touche ? 0 : this->couleur.v,
              this->touche ? 0 : this->couleur.b, this->couleur.a);

    GLfloat mat_specular[] = {0.0, 0.0, 0.0, 1.0};
    GLfloat mat_amb_diff[] = {0.5, 0.5, 0.5, 1.0};

    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_amb_diff);

    glPushMatrix(); //sauvegarde la matrice courante
    
    //on se positionne la ou la cible va tre dessin
    glTranslatef(this->position.x, this->position.y, this->position.z);

    //cout << "affichage cible : " << this->position.x << " " << this->position.y << " " << this->position.z << endl;
    //cout << this->couleur.r << " " << this->couleur.v << " " << this->couleur.b << " " << this->couleur.a << endl;

    //le poteau
    glBegin(GL_QUADS);
        glNormal3f(1, 0, 0);
        glVertex3f(0, 0, -this->potLargeur/2);
        glVertex3f(0, 0, this->potLargeur/2);
        glVertex3f(0, this->potHauteur, this->potLargeur/2);
        glVertex3f(0, this->potHauteur, -this->potLargeur/2);
    glEnd();

    //la cible
    glBegin(GL_QUADS);
        glNormal3f(1, 0, 0);
        glVertex3f(0, this->potHauteur, -this->ciLargeur/2);
        glVertex3f(0, this->potHauteur, this->ciLargeur/2);
        glVertex3f(0, this->potHauteur + this->ciHauteur, this->ciLargeur/2);
        glVertex3f(0, this->potHauteur + this->ciHauteur, -this->ciLargeur/2);
    glEnd();

    glPopMatrix(); //restaure la matrice
}

int Cible::verifCollision(Point3D A)
{
    int positionValide = 0;

    if((A.x > this->position.x - 1.0) && (A.x < this->position.x + 1.0) && (A.z > this->position.z - 1.0) && (A.z < this->position.z + 1.0))
    {
        positionValide = 1;
    }

    return positionValide;
}
